(* - Wed Jun 27 20:41:46 2001 by malcy - v0.06y *)
(* - Sat Mar 23 04:16:15 2002 by malc - lets bump it to 0.08 *)
open Xlib
open Xtypes

type placement =
  TopLeft
| TopCenter
| TopRight
| LeftCenter
| RightCenter
| BottomLeft
| BottomRight
| BottomCenter

type background =
  BColor of Xtypes.color
| BHgradient
| BVgradient
| BTile of Xpm.pixmap_data

type slit_client =
  { sc_window        : window
  ; sc_client_window : window
  ; sc_icon_window   : window
  ; mutable sc_x     : int
  ; mutable sc_y     : int
  ; mutable sc_w     : int
  ; mutable sc_h     : int
  }

let dummy_sc =
  { sc_window        = 0
  ; sc_client_window = 0
  ; sc_icon_window   = 0
  ; sc_x             = 0
  ; sc_y             = 0
  ; sc_w             = 0
  ; sc_h             = 0
  }

let dpyname =
  (* - Sat Nov 18 20:34:28 2000 by malc - Ugly workaround *)
  try
    let name = Sys.getenv "DISPLAY" in
    let low = Char.lowercase in
    let len = String.length name in
    if len > 4 &&
      low name.[0] = 'u' && low name.[1] = 'n' &&
      low name.[2] = 'i' && low name.[3] = 'x' then
        String.sub name 4 (len - 4)
    else
      name
  with
    Not_found -> ""

let dpy             = openDisplay dpyname
let root            = defaultRoot dpy
let colormap        = defaultColormap dpy
let screen          = defaultScreen dpy
let depth           = defaultDepth dpy
let defblack        = defaultBlack dpy
let defwhite        = defaultWhite dpy
let screen_width    = screen.scr_width
let screen_height   = screen.scr_height
let gcursor         = createFontCursor dpy XC.xc_crosshair
let mcursor         = createFontCursor dpy XC.xc_fleur
let cursor          = createFontCursor dpy XC.xc_left_ptr

let grad_from       = ref { red = 0x6600; green = 0x6600; blue = 0x6600 }
let grad_to         = ref { red = 0x9900; green = 0x9900; blue = 0x9900 }

let vertical        = ref true
let hidden          = ref true
let ontop           = ref false
let placement       = ref TopLeft
let bgtype          = ref (BColor { red = 0; green = 0; blue = 0 })
let bgtile          = ref !bgtype

let bevel_width     = ref 2
let border_width    = ref 1
let click_area_size = ref 11
let min_width       = ref 32
let min_height      = ref 32
let borderpix       = ref defwhite
let half_gradient   = ref false

let pids            = ref []
let clients         = ref []
let client_names    = ref []
let clients_order   = ref []
let nokill          = ref false
let verbose         = ref false
let wheel_restacks  = ref false
let panel           = ref false
let managed         = ref false
let delay           = ref 100.0
let desktop_index   = ref 0

let slit_input = [ ButtonPressMask
                 ; ButtonReleaseMask
                 ; EnterWindowMask
                 ; LeaveWindowMask
                 ; Button1MotionMask
                 ; SubstructureNotifyMask
                 ; SubstructureRedirectMask
                 ]

let win_input = [ StructureNotifyMask
                ; SubstructureNotifyMask
                ]

let intern_atom name = try X.internAtom dpy name true with _ -> noAtom
let netwmt =
  (* - Sat Aug 25 06:27:36 2001 by Malcy - NET_WM *)
  try
    let atom = intern_atom "_NET_SUPPORTING_WM_CHECK" in
    let name = intern_atom "_NET_WM_NAME" in
    if atom = noAtom then false else
    let prop win =
      let prop = getWholeProperty dpy win atom in
      if prop.gp_type != XA.xa_window && prop.gp_length != 1
      then
        noWindow
      else
        Xbuffer.getWindow prop.gp_value 0
    in
    let child = prop root in
    if child = noWindow then false else
    if child != prop child then false else
    true
  with _ -> false

let atom_workarea = intern_atom (if netwmt then  "_NET_WORKAREA" else "_WIN_WORKAREA")
let atom_nettype  = intern_atom "_NET_WM_WINDOW_TYPE"
let atom_netdesk  = intern_atom "_NET_CURRENT_DESKTOP"
let atom_strut    = intern_atom "_NET_WM_STRUT"
let atom_state    = intern_atom "_NET_WM_STATE"
let atom_top      = intern_atom "_NET_WM_STATE_STAYS_ON_TOP"
let atom_dock     = intern_atom "_NET_WM_WINDOW_TYPE_DOCK"
let atom_wmstate  = intern_atom "WM_STATE"

let desktop =
  (* - Wed Nov 15 04:56:26 2000 by Malcy - Bah *)
  (* - Sat Aug 25 06:25:57 2001 by Malcy - finally read NET_WM specs, and voila... *)
  try
    if netwmt then
      let atom_desktop = intern_atom "_NET_WM_WINDOW_TYPE_DESKTOP" in
      if atom_nettype = noAtom || atom_desktop = noAtom then noWindow else
      let rec f = function
          x :: _ when
            try
              let prop = getWholeProperty dpy x atom_nettype in
              prop.gp_type = XA.xa_atom && prop.gp_length = 1 &&
              Xbuffer.getAtom prop.gp_value 0 = atom_desktop
            with _ -> false -> x
        | x :: xs ->
            let children = (X.queryTree dpy x).qt_subwindows in
            if children = [] then f xs else
            (try ignore (f children); x with Not_found -> f xs)
        | [] -> raise Not_found
      in f (X.queryTree dpy root).qt_subwindows
    else
      let atom_winrole = intern_atom "WINDOW_ROLE" in
      if atom_winrole = noAtom then noWindow else
      let desktop_p win =
        try
          Icccm.getStringProperty dpy win atom_winrole = "desktop"
        with _ -> false
      in
      List.find desktop_p (X.queryTree dpy root).qt_subwindows
  with _ -> noWindow

let get_desktop () =
  try
    if atom_netdesk != noAtom then
      let prop = getWholeProperty dpy root atom_netdesk in
      if
        prop.gp_type != XA.xa_cardinal ||
        prop.gp_format != 4 ||
        prop.gp_length != 1
      then
        0
      else
        Xbuffer.getCard32 prop.gp_value 0
    else 0
  with _ -> 0

let get_workarea () =
  (* - Wed Nov  8 21:36:48 2000 by Malcy - Get WM workarea dimensions  *)
  try
    if atom_workarea = noAtom then (0, 0, screen_width, screen_height) else
    let prop = getWholeProperty dpy root atom_workarea in
    if
      prop.gp_type != XA.xa_cardinal ||
      prop.gp_format != 4 ||
      prop.gp_length mod 4 != 0
    then
      (0, 0, screen_width, screen_height)
    else
      let desktop = !desktop_index in
      let m = if  prop.gp_length < desktop * 4 then 0 else desktop * 16 in
      let g n = Xbuffer.getInt32 prop.gp_value (n * 4 + m) in
      let x = g 0 and y = g 1 and w = g 2 and h = g 3 in
      (x, y, w, h)
  with _ -> (0, 0, screen_width, screen_height)

let initial_workarea = get_workarea ()

let get_workarea () =
  if not netwmt && !panel then initial_workarea else get_workarea ()

let parse_color s =
  (* - Wed Nov  8 21:35:53 2000 by Malcy - Make Xtypes.color from string  *)
  (* - Fri Mar 22 17:53:43 2002 by malc - Fixed color allocation by shifting? *)
  try
    let int_of_char s i =
      match Char.lowercase s.[i] with
        'a'..'f' as h -> (10 + (int_of_char h) - (int_of_char 'a'))
      | '0'..'9' as d -> (int_of_char d) - (int_of_char '0')
      | _ -> raise Not_found
    in
    if s.[0] = '#' then begin
      if String.length s != 7 then raise Not_found;
      let r = (int_of_char s 1) lsl 4 + (int_of_char s 2)
      and g = (int_of_char s 3) lsl 4 + (int_of_char s 4)
      and b = (int_of_char s 5) lsl 4 + (int_of_char s 6) in
      (X.allocColor dpy colormap (r lsl 8) (g lsl 8) (b lsl 8)).ac_color
    end else
      (X.lookupColor dpy colormap s).lc_visual
  with Not_found | XError _ ->
    raise (Arg.Bad ("bad color specifier `" ^ s ^ "'"))

let change_placement s =
  let p =
    match (String.lowercase s) with
      "topleft"      -> TopLeft
    | "topcenter"    -> vertical := false; TopCenter
    | "topright"     -> TopRight
    | "leftcenter"   -> LeftCenter
    | "rightcenter"  -> RightCenter
    | "bottomleft"   -> BottomLeft
    | "bottomright"  -> BottomRight
    | "bottomcenter" -> vertical := false; BottomCenter
    | _ -> raise (Arg.Bad ("bad placement specifier `" ^ s ^ "'")) in
  placement := p

let strip s =
  let len = (String.length s) in
  let rec skw i =
    if i = pred len then i else
    match s.[i] with
    | ' ' | '\t' -> skw (succ i)
    | _ -> i
  in let f = skw 0 in
  let rec skn i =
    if i = f then i else
    match s.[i] with
    | ' ' | '\t' -> skn (pred i)
    | _ -> i
  in
  let l = skn (pred len) in
  String.sub s f (l - f + 1)

let get_name_value line =
  let len = String.length line in
  let semi_pos = try String.index line ';' with Not_found -> len in
  let equal_pos = try String.index line '=' with Not_found -> succ len in
  if equal_pos < semi_pos then
    let name = String.sub line 0 equal_pos
    and vle = String.sub line (succ equal_pos) (semi_pos - equal_pos - 1) in
    let name = strip name
    and vle = strip vle in
    Some (name, vle)
  else None

let split s =
  (* - Wed Jun 27 16:14:55 2001 by malcy - Stolen from CDK (with minor modifications) *)
  let len = String.length s in
  let rec index_from pos =
    if pos = len then raise Not_found else
    match s.[pos] with
    | ' ' | '\t' -> pos
    | _ -> index_from (succ pos)
  and iter pos =
    try
      if pos = len then [] else
      let pos2 = index_from pos in
      (String.sub s pos (pos2 - pos)) :: (iter (pos2 + 1))
    with _ -> [String.sub s pos (len - pos)]
  in
  iter 0

let config =
  let confname = try (Sys.getenv "HOME") ^ "/.icedockrc" with _ -> "" in
  let options =
    let defb ?(t="\t\t") o s br =
      let s = if s = "" then t else t ^ s ^ " " in
      (o, Arg.Unit (fun () -> br := not !br),
       s ^ "(default: " ^ (string_of_bool !br) ^ ")")
    and defi ?(t="\t\t") o s ir =
      let s = if s = "" then "arg" ^ t else "arg" ^ t ^ s ^ " " in
      (o, Arg.Int (fun i -> ir := i),
       s ^ "(default: " ^ (string_of_int !ir) ^ ")")
    and grad r s =
      r := parse_color s;
      match !bgtype with
        BHgradient | BVgradient -> ()
      | _ -> bgtype := BHgradient
    and color_name c =
      Format.sprintf "#%02X%02X%02X" (c.red lsr 8) (c.green lsr 8) (c.blue lsr 8)
    and hv s =
      let c = Char.lowercase s.[0] in
      if c = 'h' then true else if c = 'v' then false else
      raise (Arg.Bad ("bad layout specifier `" ^ s ^ "'"))
    in
    [("-grad_from", Arg.String (grad grad_from),
      "arg\tstarting color for gradient (default: " ^ color_name !grad_from ^ ")");
     ("-grad_to", Arg.String (grad grad_to),
      "arg\t\tending color for gradient (default: " ^ color_name !grad_to ^ ")");
     ("-color", Arg.String (fun s -> bgtype := BColor (parse_color s)),
      "arg\t\tbackground color (default: black)");
     ("-placement", Arg.String change_placement,
      "arg\t[TopLeft(default),TopCenter,TopRight,LeftCenter,\n" ^
      "\t\t\t RightCenter,BottomLeft,BottomRight,BottomCenter]");
     ("-layout", Arg.String (fun s -> vertical := not (hv s)),
      "arg\t\tv[ertical](default) or h[orizontal]");
     defi "-width" "minimal dock width" min_width;
     defi "-height" "minimal dock height" min_height;
     defi ~t:"\t" "-handle_width" "" click_area_size;
     defi ~t:"\t" "-bevel_width" "" bevel_width;
     defi ~t:"\t" "-border_width" "" border_width;
     ("-black_border", Arg.Unit (fun () -> borderpix := defblack),
      "\tblack border (default white)");
     defb ~t:"\t" "-half_gradient" "" half_gradient;
     defb "-ontop" "be allways on top" ontop;
     defb "-nokill" "do not kill clients on exit" nokill;
     (*defb "-panel" "act like panel under KDE" panel;*)
     defb "-managed" "allow WM to manage IceDock's window" managed;
     ("-tile", Arg.String (fun s ->
       bgtile := !bgtype;
       bgtype := BTile (Xpm.readPixmapDataFromFile s)),
      "arg\t\tpath to tile XPM");
     defb "-verbose" "print information on stdout" verbose;
     defb "-wheel" "use mouse wheel to restack docks" wheel_restacks;
     ("-gradient", Arg.String (fun s ->
       if hv s then bgtype := BHgradient else bgtype := BVgradient),
      "arg\t\tv[ertical] or h[orizontal](default)");
     ("-delay", Arg.String (fun s -> delay := float_of_string s),
      "arg\t\thide delay in milliseconds (default: " ^
      string_of_float !delay ^ ")")]
  in
  let parse_config () =
    let client s =
      let v = split s in
      let p = List.hd v in
      client_names := (p, Array.of_list v) :: !client_names
    and order s =
      let v = split s in
      let i = List.length !clients_order in
      if not (List.mem_assoc v !clients_order) then
        clients_order := (List.rev v, i) :: !clients_order
    in
    let options = options @
      [("-client", Arg.String client, ""); ("-order", Arg.String order, "")]
    in
    try
      let boolfn vle =
        match String.lowercase vle with
          "yes" | "1" | "true" | "on"  -> true
        | "no" | "0" | "false" | "off" -> false
        | _ -> raise (Failure vle)
      in
      let process name vle =
        let name = "-" ^ String.lowercase name in
        let (_, arg, _) = List.find (fun (s, _, _) -> name = s) options in
        try
          begin match arg with
            Arg.Int fn    -> fn (int_of_string vle)
          | Arg.String fn -> fn vle
          | Arg.Unit fn   -> if boolfn vle then fn ()
          | Arg.Set r     -> r := boolfn vle
          | _ -> raise (Failure vle)
          end
        with
          Failure _ -> Format.eprintf "bad argument `%s' for option `%s'@." vle name
        | Arg.Bad s -> Format.eprintf "%s@." s
      in
      let iter_channel f channel =
        try while true do f (input_line channel) done with End_of_file -> ()
      in
      iter_channel (fun line ->
        match (get_name_value line) with
          Some (name, vle) ->
            (try process name vle with _ ->
              Format.eprintf "unknown option `%s'@." line)
        | None -> ()) (open_in confname)
    with e ->
      Format.eprintf "exception while parsing config: %s@."
        (Printexc.to_string e)
  in
  if confname != "" then parse_config ();
  clients_order := List.rev !clients_order;
  Arg.parse options (fun s ->
    Format.eprintf "Don't know what to do with `%s'@." s;
    exit 1) "Usage: icedock [options]\nOptions are:"

let check_for_other_instance =
  let rec slit_exists win =
    try
      let tree = X.queryTree dpy win in
      List.exists (fun win ->
        try
          let name = Icccm.getWM_NAME dpy win
          and clas = Icccm.getWM_CLASS dpy win in
          (name = "icedock" && clas = ["IceDock"; "icedock"])
        with _ -> false ||
        slit_exists win) tree.qt_subwindows
    with _ -> false
  in
  if slit_exists root then begin
    Format.eprintf "Another instance is running. Exiting@.";
    closeDisplay dpy;
    exit 1;
  end

let root_input =
  if netwmt then
    [PropertyChangeMask; SubstructureNotifyMask] else [SubstructureNotifyMask]

let validate_window win =
  (* - Wed Nov  8 21:37:38 2000 by Malcy - Check if window is about to be destroyed  *)
  match (checkPredEvent dpy (fun ev ->
    match ev.ev_event with
      DestroyNotifyEvent dn -> dn.Xdestroywindow.window = win
    | CreateNotifyEvent cn -> cn.Xcreatewindow.window = win
    | _ -> false) true) with
  | Some ev -> putBackEvent dpy ev; false
  | None -> true

let vsl =
  let dpf = List.find (fun dpf -> dpf.dpf_depth = depth) screen.scr_depths in
  try
    List.find (fun vsl -> vsl.vsl_class = TrueColor) dpf.dpf_visuals
  with Not_found ->
    List.find (fun vsl -> vsl.vsl_id = screen.scr_root_visual_id)
      dpf.dpf_visuals

let rgb =
  (* - Tue Nov 28 20:13:39 2000 by Malcy - Updated to handle common bgrs  *)
  let rgb000 r g b = 0
  and rgb555 r g b = ((r lsr 3) lsl 10) lor ((g lsr 3) lsl 5) lor (b lsr 3)
  and rgb565 r g b = ((r lsr 3) lsl 11) lor ((g lsr 2) lsl 5) lor (b lsr 3)
  and rgb888 r g b = (r lsl 16) lor (g lsl 8) lor b in
  let bgr555 r g b = rgb555 b g r
  and bgr565 r g b = rgb565 b g r
  and bgr888 r g b = rgb888 b g r in
  if vsl.vsl_class != TrueColor then
    rgb000
  else
    match vsl.vsl_red_mask, vsl.vsl_green_mask, vsl.vsl_blue_mask with
      0x7c00, 0x3e0, 0x1f when depth = 15 -> rgb555
    | 0x1f, 0x3e0, 0x7c00 when depth = 15 -> bgr555
    | 0xf800, 0x7e0, 0x1f when depth = 16 -> rgb565
    | 0x1f, 0x7e0, 0xf800 when depth = 16 -> bgr565
    | 0xff0000, 0xff00, 0xff when depth = 24 || depth = 32 -> rgb888
    | 0xff, 0xff00, 0xff0000 when depth = 24 || depth = 32 -> bgr888
    | _ ->
        Format.eprintf "unsupported: depth %d; masks red %x, green %x, blue %x@."
          depth vsl.vsl_red_mask vsl.vsl_green_mask vsl.vsl_blue_mask;
        rgb000

let slit =
  (* - Wed Nov  8 21:37:15 2000 by Malcy - Create slit window  *)
  (*panel := !panel && netwmt;*)
  panel := !managed || netwmt;
  let slit =
    X.createWindow dpy root 0 0 1 1 depth InputOutput vsl.vsl_id
      0
      (let cw =
        [ CWBackPixel defwhite
        ; CWBorderPixel !borderpix
        ; CWOverrideRedirect (not !panel)
        ; CWEventMask slit_input
        ] in
      if netwmt then CWCursor cursor :: cw else cw)
  in
  Icccm.setWM_NAME dpy slit "icedock";
  Icccm.setWM_CLASS dpy slit ["icedock"; "IceDock"];
  X.configureWindow dpy slit [CWBorderWidth !border_width];
  if !panel then begin
    if netwmt then begin
      Icccm.setEnum32Property dpy slit atom_nettype XA.xa_atom [atom_dock];
      if !ontop then
        Icccm.setEnum32Property dpy slit atom_state XA.xa_atom [atom_top];
    end else begin
      let hints = Icccm.newWM_HINTS () in
      Icccm.setWM_HINTS dpy slit { hints with input = Some false };

      let atom = X.internAtom dpy "_MOTIF_WM_HINTS" false in
      let hints = Xbuffer.newString 5 in
      Xbuffer.setCard32 hints 0 7;
      Xbuffer.setCard32 hints 4 0;
      Xbuffer.setCard32 hints 8 (if !border_width > 0 then 2 else 0);
      Xbuffer.setEnum32 hints 12 0;
      X.changeProperty dpy slit PropModeReplace atom atom 4 hints
    end;
  end;

  if !verbose then begin
    Format.printf "Icedock window = %08x@." slit;
  end;
  slit

let blend ?(t=0.5) ?(r=255.0) ?(g=255.0) ?(b=255.0) sr sg sb =
  rgb
    (int_of_float ((sr *. t) +. (r *. (1.0 -. t))))
    (int_of_float ((sg *. t) +. (g *. (1.0 -. t))))
    (int_of_float ((sb *. t) +. (b *. (1.0 -. t))))

let gradient_lines w =
  (* - Wed Nov  8 21:38:30 2000 by Malcy - Make gradient lines *)
  let nl = Array.create (w * 3) 0
  and ll = Array.create (w * 3) 0
  and dl = Array.create (w * 3) 0
  and c x = float_of_int (x lsr 8) in
  let gfrom = !grad_from and gto = !grad_to in
  let fr = ref (c gfrom.red)
  and fg = ref (c gfrom.green)
  and fb = ref (c gfrom.blue)
  and tr = c gto.red
  and tg = c gto.green
  and tb = c gto.blue in

  let wf = (if !half_gradient then 0.5 else 1.0) *. float_of_int w in
  let dr = (tr -. !fr) /. wf
  and dg = (tg -. !fg) /. wf
  and db = (tb -. !fb) /. wf in

  let blendl = blend in
  let blendd = blend ~r:0.0 ~g:0.0 ~b:0.0 in
  let xmax = if !half_gradient then (w / 2) + (w mod 2) else w in
  for i = 0 to pred xmax do
    nl.(i) <- rgb (int_of_float !fr) (int_of_float !fg) (int_of_float !fb);
    ll.(i) <- blendl !fr !fg !fb;
    dl.(i) <- blendd !fr !fg !fb;
    fr := !fr +. dr;
    fg := !fg +. dg;
    fb := !fb +. db
  done;
  if !half_gradient then begin
    let r = pred w in
    for i = xmax to r do
      nl.(i) <- nl.(r - i);
      ll.(i) <- ll.(r - i);
      dl.(i) <- dl.(r - i)
    done
  end;
  (nl, ll, dl)

let gradient horizontal cw ch w h =
  let nl, ll, dl = gradient_lines (if horizontal then w else h) in
  let pixmap = Zpixmap.create dpy w h depth in
  for y = 0 to pred h do
    for x = 0 to pred w do
      let i = if horizontal then x else y in
      Zpixmap.setPixel pixmap x y nl.(i)
    done
  done;
  let loop init max inc f =
    let rec g i = if i < max then (f i; g (i + inc)) else () in
    g init
  in
  if ch = 0 then
    let lc = ll.(0) and dc = dl.(if horizontal then w else h) in
    for y = 1 to h - 3 do
      loop 1 (pred cw) 3 begin fun x ->
        Zpixmap.setPixel pixmap x y lc;
        Zpixmap.setPixel pixmap (succ x) (succ y) dc
      end
    done
  else
    loop 1 (pred ch) 3 begin fun y ->
      for x = 1 to w - 3 do
        let i = if horizontal then x else y in
        Zpixmap.setPixel pixmap x y ll.(i);
        Zpixmap.setPixel pixmap (succ x) (succ y) dl.(i)
      done
    end;
  pixmap

let background =
  let slit_pixmap = ref noPixmap in
  let grad f _ _ cw ch w h =
    let pixmap = X.createPixmap dpy slit w h depth in
    let gc = X.createGC dpy pixmap [] in
    let zpixmap = f cw ch w h in
    Zpixmap.putImage dpy pixmap gc 0 0 zpixmap;
    X.freeGC dpy gc;
    X.changeWindowAttributes dpy slit [CWBackPixmap pixmap];
    if !slit_pixmap != noPixmap then X.freePixmap dpy !slit_pixmap;
    slit_pixmap := pixmap

  and tile data fill _ _ cw ch w h =
    let (width, height, depth, pixmap1, pixmap2) = data in
    let zpixmap = Zpixmap.getImage dpy pixmap1 0 0 width height 0xffffffff in
    let pixmap = X.createPixmap dpy slit w h depth in
    let gc = X.createGC dpy pixmap [] in

    let n = List.length !clients in

    begin match fill with
      `gradient (zh, zv) ->
        fillRectangle dpy pixmap gc 0 0 w h;
        Zpixmap.putImage dpy pixmap gc 0 0 (if cw = 0 then zh else zv)
    | `color c ->
        setForeground dpy gc c;
        fillRectangle dpy pixmap gc 0 0 w h
    end;

    if cw = 0 then begin
      let y = ref ch in
      for i = 0 to pred n do
        Zpixmap.putImage dpy pixmap gc !bevel_width !y zpixmap;
        y := !y + height + !bevel_width;
      done;
    end else begin
      let x = ref cw in
      for i = 0 to pred n do
        Zpixmap.putImage dpy pixmap gc !x !bevel_width zpixmap;
        x := !x + width + !bevel_width;
      done;
    end;

    X.freeGC dpy gc;
    X.changeWindowAttributes dpy slit [CWBackPixmap pixmap];
    if !slit_pixmap != noPixmap then X.freePixmap dpy !slit_pixmap;
    slit_pixmap := pixmap

  and dummy _ _ _ _ _ _ = () in
  match !bgtype with
    BColor c ->
      (try
        let acr = (X.allocColor dpy colormap c.red c.green c.blue) in
        X.changeWindowAttributes dpy slit [CWBackPixel acr.ac_pixel]
      with _ -> ()); dummy
  | BTile data ->
      let (width, height, _, _, _) as p =
        Xpm.createPixmapFromData dpy slit colormap depth data
      in
      begin match !bgtile with
        BColor c ->
          let pixel =
            try
              (X.allocColor dpy colormap c.red c.green c.blue).ac_pixel
            with _ -> 0
          in
          tile p (`color pixel)
      | BHgradient | BVgradient ->
          let c = !click_area_size and b2 = !bevel_width * 2 in
          let gt = !bgtile = BHgradient in
          let zh = gradient gt 0 c (width + b2) c
          and zv = gradient gt c 0 c (height + b2) in
          tile p (`gradient (zh, zv))
      | _ -> tile p (`color defblack)
      end
  | BHgradient -> grad (gradient true)
  | BVgradient -> grad (gradient false)

let bounds ?(placement = !placement) ?(vert = !vertical) workarea =
  (* - Mon Nov 20 20:18:41 2000 by Malcy - Moved hide logic here  *)
  (* - Fri May 10 14:58:49 2002 by malc - rewritten *)
  let (wl, wt, ww, wh) = workarea in

  let bevel_w = !bevel_width
  and bw2 = 2 * !border_width in

  let n = List.length !clients in
  let vert =
    if !hidden then
      match placement with
      | TopCenter | BottomCenter -> vertical := true; true
      | LeftCenter | RightCenter -> vertical := false; false
      | _ -> vert
    else vert
  in

  let w, h =
    match !bgtype with
      BTile (width, height, _, _) ->
        if vert then
          width, height * n
        else
          width * n, height

    | otherwise ->
        let initial =
          if vert then (!min_width, 0) else (0, !min_height)
        in
        List.fold_left begin fun (w, h) (_, sc) ->
          if vert then
            max w sc.sc_w, h + sc.sc_h
          else
            w + sc.sc_w, max h sc.sc_h
        end initial !clients
  in
  let slit_w = if vert then w + 2 * bevel_w else w + n * bevel_w
  and slit_h = if vert then h + n * bevel_w else h + 2 * bevel_w in

  let slit_w, slit_h =
    if !hidden then
      (if vert then slit_w, 0 else 0, slit_h)
    else slit_w, slit_h
  in

  let cw = if vert then 0 else !click_area_size
  and ch = if vert then !click_area_size else 0 in
  let full_w = slit_w + cw
  and full_h = slit_h + ch in

  let (x, y) =
    let w = full_w + bw2 and h = full_h + bw2 in
    let xm = ww - w
    and ym = wh - h
    and xc = (ww - w) / 2
    and yc = (wh - h) / 2 in
    let (x, y) =
      (match placement with
         TopLeft      -> ( 0,  0)
       | TopCenter    -> (xc,  0)
       | TopRight     -> (xm,  0)
       | LeftCenter   -> ( 0, yc)
       | RightCenter  -> (xm, yc)
       | BottomLeft   -> ( 0, ym)
       | BottomRight  -> (xm, ym)
       | BottomCenter -> (xc, ym)
      ) in (wl + x, wt + y)
  in
  (x, y, cw, ch, full_w, full_h)

let reconfigure () =
  (* - Wed Nov  8 21:38:48 2000 by Malcy - Reconfigure slit and docked windows  *)
  let (x, y, cw, ch, sw, sh) = bounds (get_workarea ()) in
  moveResizeWindow dpy slit x y sw sh;

  background x y cw ch sw sh;
  X.clearWindow dpy slit;

  let send_event e =
    try
      moveResizeWindow dpy e.sc_window e.sc_x e.sc_y e.sc_w e.sc_h;
      X.mapWindow dpy e.sc_window;
      let event = ConfigureNotifyEvent {
        Xconfigure.event             = e.sc_window;
        Xconfigure.window            = e.sc_window;
        Xconfigure.x                 = e.sc_x;
        Xconfigure.y                 = e.sc_y;
        Xconfigure.width             = e.sc_w;
        Xconfigure.height            = e.sc_h;
        Xconfigure.border_width      = 0;
        Xconfigure.above_sibling     = slit;
        Xconfigure.override_redirect = false;
      } in
      X.sendEvent dpy e.sc_window true [StructureNotifyMask] event
    with XError _ -> ()
  in
  let conf =
    match !bgtype with
      BTile (width, height, _, _) ->
        let f (x, y) (_, e) =
          if !vertical then
            let x = (sw - e.sc_w) / 2 in
            let t = (height - e.sc_h) / 2 in
            e.sc_x <- x;
            e.sc_y <- y + t;
            send_event e;
            (x, y + height + !bevel_width)
          else
            let y = (sh - e.sc_h) / 2 in
            let l = (width - e.sc_w) / 2 in
            e.sc_x <- x + l;
            e.sc_y <- y;
            send_event e;
            (x + width + !bevel_width, y)
        in
        f
    | otherwise ->
        let f (x, y) (_, e) =
          if !vertical then
            let x = (sw - e.sc_w) / 2 in
            e.sc_x <- x;
            e.sc_y <- y;
            send_event e;
            (x, y + e.sc_h + !bevel_width)
          else
            let y = (sh - e.sc_h) / 2 in
            e.sc_x <- x;
            e.sc_y <- y;
            send_event e;
            (x + e.sc_w + !bevel_width, y)
        in
        f
  in
  let _ = List.fold_left conf (cw, ch) !clients in
  Gc.compact ()                         (* extremly elegant *)

let reconfigure_light () =
  (* - Wed Nov  8 21:38:48 2000 by Malcy - Reconfigure slit and docked windows  *)
  let (x, y, cw, ch, sw, sh) = bounds (get_workarea ()) in
  let h_conf (x, y) (win, e) =
    let y = (sh - e.sc_h) / 2 in
    e.sc_x <- x;
    e.sc_y <- y;
    moveResizeWindow dpy e.sc_window e.sc_x e.sc_y e.sc_w e.sc_h;
    (x + e.sc_w + !bevel_width, y)
  and v_conf (x, y) (win, e) =
    let x = (sw - e.sc_w) / 2 in
    e.sc_x <- x;
    e.sc_y <- y;
    moveResizeWindow dpy e.sc_window e.sc_x e.sc_y e.sc_w e.sc_h;
    (x, y + e.sc_h + !bevel_width)
  and h_tile_conf width (x, y) (win, e) =
    let y = (sh - e.sc_h) / 2 in
    let l = (width - e.sc_w) / 2 in
    e.sc_x <- x + l;
    e.sc_y <- y;
    moveResizeWindow dpy e.sc_window e.sc_x e.sc_y e.sc_w e.sc_h;
    (x + width + !bevel_width, y)
  and v_tile_conf height (x, y) (win, e) =
    let x = (sw - e.sc_w) / 2 in
    let t = (height - e.sc_h) / 2 in
    e.sc_x <- x;
    e.sc_y <- y + t;
    moveResizeWindow dpy e.sc_window e.sc_x e.sc_y e.sc_w e.sc_h;
    (x, y + height + !bevel_width)
  in
  let conf =
    match !bgtype with
      BTile (width, height, _, _) ->
        if !vertical then v_tile_conf height else h_tile_conf width
    | otherwise -> if !vertical then v_conf else h_conf
  in
  ignore (List.fold_left conf (cw, ch) !clients)

let select_input win l = ignore (Xlib.selectInput dpy win l)

let insert_in_order sc win =
  let x = (win, sc)
  and order sc =
    try
      List.assoc (Icccm.getWM_CLASS dpy sc.sc_client_window) !clients_order
    with _ -> max_int in
  let u = order sc in
  let rec f accu = function
      ((_, scx) as y) :: xs ->
        let v = order scx in
        if u = v then List.rev (x :: y :: accu) @ xs
        else if u < v then List.rev (y :: x :: accu) @ xs
        else f (y :: accu) xs
    | [] -> List.rev (x :: accu)
  in clients := f [] !clients

let add_client win =
  (* - Wed Nov  8 21:39:04 2000 by Malcy - Add client to slit  *)
  select_input win [];
  select_input slit [];
  select_input root [];

  let client =
    let icon =
      let name = try Icccm.getWM_NAME dpy win with Not_found -> "" in
      if name = "LicqWharf" then
        noWindow
      else
        try
          let prop = getWholeProperty dpy win XA.xa_wm_hints in
          if (prop.gp_format <> 4) || (prop.gp_type <> XA.xa_wm_hints) then
            noWindow
          else
            let flag = Xbuffer.getCard32 prop.gp_value 0 in
            if 0 = (flag land 8) then noWindow
            else (Xbuffer.getEnum32 prop.gp_value 16)
        with Not_found -> noWindow
    in
    if icon = noWindow then
      { dummy_sc with sc_window = win
      ; sc_client_window = win
      ; sc_icon_window = noWindow }
    else
      { dummy_sc with sc_window = icon
      ; sc_client_window = win
      ; sc_icon_window = icon }
  in
  let win = client.sc_window in
  let g = X.getGeometry dpy win in

  client.sc_w <- g.gg_width;
  client.sc_h <- g.gg_height;

  X.reparentWindow dpy win slit 0 0;
  X.changeSaveSet dpy win InsertResource;

  select_input root root_input;
  select_input slit slit_input;
  select_input win win_input;

  insert_in_order client win

let string_of_state = function
    WithdrawnState -> "withdrawn"
  | IconicState -> "iconic"
  | NormalState -> "normal"
  | UndefinedState -> "undefined"

let string_of_category = function
    `withdrawn -> "`withdrawn"
  | `normal    -> "`normal"
  | `iconic    -> "`iconic"
  | `undefined -> "`undefined"
  | `nostate   -> "`nostate"
  | `nohints   -> "`nohints"

let preadd_client win =
  try
    let state, icon = Icccm.getWM_STATE dpy win in
    if !verbose then
      Format.printf "preadd %08x %s@." win (string_of_state state);

    let tree = X.queryTree dpy win in   (* This needed for WM to unmanage window *)
    if tree.qt_parent != root &&
      (X.getWindowAttributes dpy win).gwa_map_state = IsUnmapped then
      raiseWindow dpy win;

    if state != WithdrawnState then Icccm.withdrawWindow dpy win root
  with Not_found ->
    if !verbose then
      Format.printf "create %08x nostate@." win

let flush_events events =
  (* - Fri Jul 20 20:28:15 2001 by Malcy - All hail tail-call *)
  let rec f () =
    match (checkTypedWindowEvent dpy slit events false) with
      Some _ -> f ()
    | None -> ()
  in f ()

let flush_events' win events =
  (* - Fri Jul 20 20:28:15 2001 by Malcy - All hail tail-call *)
  let rec f () =
    match (checkTypedWindowEvent dpy win events false) with
      Some _ -> f ()
    | None -> ()
  in f ()

let remove_client destroyed (win, sc) =
  if not destroyed then begin
    if sc.sc_icon_window != noWindow then begin
      if validate_window win then begin
        select_input slit [];
        select_input win [];
        select_input root [];

        X.changeSaveSet dpy win DeleteResource;
        X.reparentWindow dpy win sc.sc_client_window 0 0;
        X.reparentWindow dpy sc.sc_client_window root 0 0;
        X.unmapWindow dpy win;
        X.mapWindow dpy sc.sc_client_window;

        select_input slit slit_input;
        select_input root root_input;
      end;
    end else
      let win = sc.sc_window in
      if validate_window win then begin
        select_input slit [];
        select_input win [];
        select_input root [];

        X.changeSaveSet dpy win DeleteResource;
        X.unmapWindow dpy win;
        X.reparentWindow dpy win root 0 0;
        X.mapWindow dpy win;

        select_input slit slit_input;
        select_input root root_input;
      end;
  end;
  clients := List.remove_assoc win !clients

let remove_clients () =
  if not !nokill then begin
    List.iter (fun pid ->
      if !verbose then Format.eprintf "sending SIGHUP to %d@." pid;
      try Unix.kill pid Sys.sighup with _ -> ()) !pids;
    List.iter (fun pid ->
      if !verbose then Format.eprintf "sending SIGTERM to %d@." pid;
      try Unix.kill pid Sys.sigterm with _ -> ()) !pids;
  end;
  let clients = !clients in
  try List.iter (remove_client false) clients with _ -> ()

let pwin win =
  Format.eprintf "window %08x %s@." win
      (try Icccm.getWM_NAME dpy win with Not_found -> "noname")

let base w = w land (lnot dpy.resource_mask)

let categorize_hints win =
  try
    let hints = Icccm.getWM_HINTS dpy win XA.xa_wm_hints in
    let state =
      match hints.initial_state with
        Some state ->
          begin match state with
            WithdrawnState -> `withdrawn
          | NormalState -> `normal
          | UndefinedState -> `undefined
          | IconicState -> `iconic
          end
        (*Some state when state = WithdrawnState -> `withdrawn*)
      (*| Some state -> `somestate*)
      | None -> `nostate
    in
    state
  with Not_found -> `nohints

let withdrawn_p win =
  win != slit && win != root && win != desktop &&
  categorize_hints win = `withdrawn

let rec find_withdrawn win =
  let tree = try X.queryTree dpy win
  with _ -> { qt_root = root; qt_parent = noWindow; qt_subwindows = [] } in
  List.fold_left
    (fun accu win ->
      try
        let hints = Icccm.getWM_HINTS dpy win XA.xa_wm_hints in
        let dockt =
            (*hints.icon_window != noWindow ||*)
          match hints.initial_state with
            Some state when state = WithdrawnState ->
              true
          | _ -> false
        in if dockt then win :: accu else accu
      with Not_found -> accu @ find_withdrawn win)
    [] tree.qt_subwindows

let motion_gc = X.createGC dpy root
                  [ GCline_width !border_width
                  ; GCforeground defwhite
                  ; GCfonction GXxor
                  ; GCsubwindow_mode IncludeInferiors
                  ; GCgraphics_exposures false
                  ]

type motion_state =
  { ms_rect      : (int * int * int * int)
  ; ms_placement : placement
  ; ms_vertical  : bool
  }

let point_in_rect x y l t w h =
  (x >= l && x <= (l + w)) &&
  (y >= t && y <= (t + h))

let point_in_geom x y g = point_in_rect x y g.gg_x g.gg_y g.gg_width g.gg_height

let handle_dragslit state mn =
  let rx = mn.Xmotion.x_root
  and ry = mn.Xmotion.y_root in
  let (gx, gy, gw, gh) = state.ms_rect in
  let (wl, wt, ww, wh) as wa = get_workarea () in
  let wxc = (ww / 2)
  and wyc = (wh / 2) in
  let b = !border_width in
  let b2 = b / 2 in
  let w2 = (gw / 2)
  and h2 = (gh / 2) in
  let layout =
    [ (TopLeft,      wl + w2,  wt + h2,  (wl, wt))
    ; (TopCenter,    wl + wxc, wt + h2,  (wxc - w2, wt))
    ; (TopRight,     ww - w2,  wt + h2,  (ww - gw - b + wl, wt))
    ; (LeftCenter,   wl + w2,  wt + wyc, (wl, wyc - h2))
    ; (RightCenter,  ww - w2,  wt + wyc, (ww - gw - b + wl, wyc - h2))
    ; (BottomLeft,   wl + w2,  wh - h2,  (wl, wh - gh - b + wt))
    ; (BottomCenter, wl + wxc, wh - h2,  (wxc - w2, wh - gh - b + wt))
    ; (BottomRight,  ww - w2,  wh - h2,  (ww - gw - b + wl, wh - gh - b + wt))
    ]
  in
  let distsq x y rx ry =
    let sqr a = a * a in
    sqr (rx - x) + sqr (ry - y)
  in
  let defs =
    List.fold_left
      (fun (lp, ld, lpos) (p, x, y, pos) ->
         let d = distsq x y rx ry in
         if d < ld then (p, d, pos) else (lp, ld, lpos))
      (state.ms_placement, max_int, (0, 0))
      layout
  in
  let (p, x, y, w, h, swap) =
    let (p, d, (x, y)) = defs in
    let sx, sy, _, _, sw, sh =
      bounds ~placement:p ~vert:(not state.ms_vertical) wa
    in
    let pird = point_in_rect rx ry  x  y gw gh
    and pirs = point_in_rect rx ry sx sy sw sh in
    let retd = (p, x + b2,  y + b2, gw, gh, false)
    and rets = (p, sx + b2, sy + b2, sw + b, sh + b, true) in
    if pird then retd else if pirs then rets else retd;
  in

  if swap || p != state.ms_placement then begin
    let (ox, oy, ow, oh) = state.ms_rect in
    drawRectangle dpy root motion_gc ox oy ow oh;
    drawRectangle dpy root motion_gc x y w h;
    { ms_placement = p
    ; ms_vertical = if swap then not state.ms_vertical else state.ms_vertical
    ; ms_rect = (x, y, w, h)
    }
  end else state

let hide =
  let hvert = ref !vertical in
  fun () ->
    if !hidden then
      vertical := !hvert
    else
      hvert := !vertical;
    hidden := not !hidden;
    reconfigure ()

let candrag_condition xb =
  (xb.Xbutton.detail = 1) &&
  (*(xb.Xbutton.state = 0) &&*)
  (!click_area_size >
   (if !vertical then
     xb.Xbutton.y_event
   else
     xb.Xbutton.x_event))

let swallow any =
  let swin = Xmu.selectWindow dpy root gcursor in
  if swin != noWindow && swin != root then
    let win = Xmu.clientWindow dpy swin in
    if (base swin != base win && win != slit && win != root && win != desktop) then
      if any || withdrawn_p win then preadd_client win

let swap_clients c1 c2 =
  let rec swap t = function
      [] -> []
    | (w, c) as e :: cs when c = c1 && t ->
        (c2.sc_window, c2) :: e :: swap false cs
    | (w, c) as e :: cs when c = c2 && t ->
        (c1.sc_window, c1) :: e :: swap false cs
    | (w, c) as e :: cs when c = c1 || c = c2 -> swap t cs
    | c :: cs -> c :: swap t cs
  in
  swap true !clients

let insert_before c1 c2 =
  let ie = (c1.sc_window, c1) in
  List.fold_left
    (fun l ((w, c) as e) -> if c = c2 then ie :: e :: l else e :: l)
    [] (List.rev !clients)

let insert_after c1 c2 =
  let ie = (c1.sc_window, c1) in
  List.fold_left
    (fun l ((w, c) as e) -> if c = c2 then e :: ie :: l else e :: l)
    [] (List.rev !clients)

type mode =
  MNone
| MCanDragSlit of float                 (* hack hack *)
| MCanDragDock
| MDragSlit of motion_state
| MDragDock of slit_client

let first = List.hd
let rec last = function
    x :: [] -> x
  | x :: xs -> last xs
  | [] -> raise Not_found

let client_for_point x y client mapped =
  match !bgtype with
    BTile (width, height, _, _) ->
      let index = 
        if !vertical then
          (y - !click_area_size) / (height + !bevel_width)
        else
          (x - !click_area_size) / (width + !bevel_width)
      in
      List.nth !clients index
  | otherwise ->
      let v_p (_, c) =
        y >= c.sc_y &&
        y < c.sc_y + c.sc_h + !bevel_width
      and h_p (_, c) =
        x >= c.sc_x &&
        x < c.sc_x + c.sc_w + !bevel_width
      in
      if !clients = [] || (mapped && (if !vertical then v_p else h_p) client) then
        client
      else
        let (_, fc) as f = first !clients
        and (_, lc) as l = last !clients in
        if !vertical then
          let min = fc.sc_y and max = lc.sc_y + lc.sc_h + !bevel_width in
          if y < min then f else if y >= max then l else List.find v_p !clients
        else
          let min = fc.sc_x and max = lc.sc_x + lc.sc_w + !bevel_width in
          if x < min then f else if x >= max then l else List.find h_p !clients

let handle_dragdock mn sc =
  let insert_before_after dsc =
    let x = mn.Xmotion.x_event and y = mn.Xmotion.y_event in
    let after_p =
      if !vertical then
        y > dsc.sc_y + (dsc.sc_h / 2)
      else
        x > dsc.sc_x + (dsc.sc_w / 2)
    in
    (if after_p then insert_after else insert_before) sc dsc
  in
  match mn.Xmotion.child with
    no when no = noWindow ->
      let win = sc.sc_window in
      let lc = (win, sc) in
      let g = X.getGeometry dpy slit in
      let t = List.mem_assoc win !clients in

      let inside =
        if !managed then
          point_in_geom mn.Xmotion.x_event mn.Xmotion.y_event g
        else
          point_in_geom mn.Xmotion.x_root mn.Xmotion.y_root g
      in

      if inside then begin
        let x = mn.Xmotion.x_event and y = mn.Xmotion.y_event in
        let (dwin, dsc) as dc = client_for_point x y lc t in
        let nclients =
          match t with
            true when dwin != win -> swap_clients sc dsc
          | false when dwin != win -> insert_before_after dsc
          | false -> (win, sc) :: !clients
          | _ -> []
        in
        if not t || dwin != win then begin
          clients := nclients;

          if t then reconfigure_light () else reconfigure ();
          X.warpPointer dpy noWindow 0 0 0 0
            sc.sc_window (sc.sc_w / 2) (sc.sc_h / 2);
	  flush_events [MotionNotify];
        end
      end else
        if t then begin
          clients := List.remove_assoc win !clients;
          X.unmapWindow dpy sc.sc_window;
          reconfigure ();
	  flush_events [MotionNotify];
        end;
      MDragDock sc
  | win when win = sc.sc_window -> MDragDock sc
  | win ->
      let dsc = List.assoc win !clients in
      let t = not (List.mem_assoc sc.sc_window !clients) in
      if t then
        clients := insert_before_after dsc
      else
        clients := swap_clients sc dsc;
      if t then reconfigure () else reconfigure_light ();
      X.warpPointer dpy noWindow 0 0 0 0
        sc.sc_window (sc.sc_w / 2) (sc.sc_h / 2);
      flush_events [MotionNotify];
      MDragDock sc

let lower_slit () =
  if !panel or desktop = noWindow then
    lowerWindow dpy slit
  else
    try
      X.configureWindow dpy slit [CWStackMode Above; CWSibling desktop]
    with
      XError x when x.err_code = BadWindow ->
        Format.eprintf "%s@." (xerror_to_string x);
        failwith "Desktop window probably destroyed, you must restart IceDock"

let maybe_remove_client destroyed win =
  try
    let sc = (List.assoc win !clients) in
    remove_client destroyed (win, sc);
    reconfigure ()
  with Not_found -> ()

(***********************************************************************)
(* X Events                                                            *)
(***********************************************************************)
let handle_buttonpress bp mode =
  match mode with
    MNone when bp.Xbutton.child != noWindow -> MCanDragDock
  | MNone when candrag_condition bp -> MCanDragSlit (Xtypes.real_time bp.Xbutton.time)
  | MNone when bp.Xbutton.detail = 3 ->
      if bp.Xbutton.state land controlMask != 0 then
        let pclients = find_withdrawn root in
        let nclients = List.filter
                         (fun win ->
                           not (List.exists
                                  (fun (_, c) -> c.sc_client_window = win)
                                  !clients))
                         pclients in
        if !verbose then
          Format.printf "%d clients found@." (List.length nclients);

        if nclients != [] then List.iter preadd_client nclients;
      else
        swallow (0 != bp.Xbutton.state land mod1Mask);
      mode
  | MNone when !wheel_restacks && bp.Xbutton.detail = 4 && !clients != [] ->
      clients := List.tl !clients @ [List.hd !clients];
      reconfigure_light ();
      mode
  | MNone when !wheel_restacks && bp.Xbutton.detail = 5 && !clients != [] ->
      let tail = List.rev (List.tl (List.rev !clients))
      and head = last !clients in
      clients := head :: tail;
      reconfigure_light ();
      mode
  | MNone when
      !verbose &&
      bp.Xbutton.detail = 1 && bp.Xbutton.state land mod1Mask != 0 ->
        List.iter (fun (win, sc) ->
          Format.printf "%dx%d+%d+%d\n" sc.sc_w sc.sc_h sc.sc_x sc.sc_y;
          Format.printf "%08x %08x@." sc.sc_client_window sc.sc_window)
          !clients;
        mode
  | _ -> mode

let handle_buttonrelease br mode =
  match mode with
    MDragSlit state ->
      X.changeWindowAttributes dpy slit [CWCursor cursor];
      let (x, y, w, h) = state.ms_rect in
      drawRectangle dpy root motion_gc x y w h;
      if !placement != state.ms_placement ||
        !vertical != state.ms_vertical then begin
	  flush_events [LeaveNotify; EnterNotify];
          placement := state.ms_placement;
          vertical  := state.ms_vertical;
          reconfigure ();
        end;
      MNone
  | MDragDock sc ->
      if not (List.mem_assoc sc.sc_window !clients) then
        remove_client false (sc.sc_window, sc);
      X.changeWindowAttributes dpy slit [CWCursor cursor];
      X.changeWindowAttributes dpy sc.sc_window [CWCursor cursor];
      MNone
  | _ when br.Xbutton.detail = 2 &&
      let g = X.getGeometry dpy slit in
      ((br.Xbutton.x_event > 0) && (br.Xbutton.x_event < g.gg_width)) &&
      ((br.Xbutton.y_event > 0) && (br.Xbutton.y_event < g.gg_height)) ->
        raise Exit
  | MCanDragDock -> MNone
  | MCanDragSlit time ->
      let cur = (Xtypes.real_time br.Xbutton.time) in
      if cur -. time > !delay then hide ();
      MNone
  | _ -> mode

let handle_motion mn mode =
  match mode with
    MDragSlit state -> MDragSlit (handle_dragslit state mn);
  | MDragDock sc -> handle_dragdock mn sc

  | MCanDragSlit _ ->
      X.changeWindowAttributes dpy slit [CWCursor mcursor];
      let sx, sy, _, _, sw, sh = bounds (get_workarea ()) in
      let b = !border_width in
      let b2 = b / 2 in
      let sx = sx + b2 and sy = sy + b2 in
      let sw = sw + b and sh = sh + b in
      let (x, y, w, h) as rect = (sx, sy, sw, sh) in
      drawRectangle dpy root motion_gc x y w h;
      let state = handle_dragslit { ms_placement = !placement;
                                    ms_vertical = !vertical;
                                    ms_rect = rect
                                  } mn in
      MDragSlit state

  | MCanDragDock when mn.Xmotion.child != noWindow ->
      let win = mn.Xmotion.child in
      let sc = List.assoc win !clients in
      X.warpPointer dpy noWindow 0 0 0 0
        win (sc.sc_w / 2) (sc.sc_h / 2);
      X.changeWindowAttributes dpy slit [CWCursor mcursor];
      X.changeWindowAttributes dpy win [CWCursor mcursor];
      MDragDock sc
  | othwerwise -> mode

let handle_enter en mode =
  match (checkTypedWindowEvent dpy slit [LeaveNotify] false) with
  | Some _ -> mode
  | None ->
      begin match mode with
        MNone when en.Xcrossing.detail != NotifyInferior ->
          raiseWindow dpy slit;
          mode
      | _ -> mode
      end

let handle_leave ln mode =
  match (checkTypedWindowEvent dpy slit [EnterNotify] false) with
  | Some _ -> mode
  | None ->
      begin match mode with
        MNone when ln.Xcrossing.detail != NotifyInferior ->
          if not !ontop then lower_slit ();
          mode
      | _ -> mode
      end

let handle_unmap ev un mode =
  let win = un.Xunmap.window in
  maybe_remove_client false win;
  mode

let start_clients () =
  let a v = String.concat " " (List.tl (Array.to_list v)) in
  List.iter (fun (p, v) ->
    if !verbose then begin
      Format.printf "starting %s %s@." p (a v);
    end;
    try
      let pid = Unix.fork () in
      if pid = 0 then
        try
          Unix.execvp p v
        with e ->
	  Format.eprintf "exception %s\nwhile spawning client `%s %s'\nproceeding@."
    	    (Printexc.to_string e) p (a v);
	  exit 1
      else pids := pid :: !pids
    with e ->
      Format.eprintf "exception %s\nwhile spawning client `%s %s'\nproceeding@."
        (Printexc.to_string e) p (a v))
    !client_names

let root_event ev =
  let module X = Xcreatewindow in
  match ev.ev_event with
    CreateNotifyEvent e when
      not e.X.override_redirect &&
      e.X.parent = root &&
      base e.X.window != base e.X.parent
    ->
      let win = e.X.window in
      select_input win [PropertyChangeMask];
      let category = categorize_hints win in

      if !verbose then
        Format.printf "create %08x %s@." win (string_of_category category);

      if category = `withdrawn then preadd_client win

  | ReparentNotifyEvent e when e.Xreparent.parent = root ->
      let win = e.Xreparent.window in
      let category = categorize_hints win in
      if category = `withdrawn then begin

        if !verbose then
          Format.printf "reparent %08x@." e.Xreparent.window;

        add_client win;
        reconfigure ()
      end

  | otherwise -> ()

let win_event ev =
  let module P = Xproperty in
  match ev.ev_event with
    PropertyNotifyEvent e when
      e.P.state = PropertyNewValue &&
      e.P.atom = XA.xa_wm_hints
    ->
      let win = e.P.window in
      let category = categorize_hints win in

      if !verbose then
        Format.printf "newhints %08x %s@." win (string_of_category category);

      if category = `withdrawn then
        preadd_client win
      else begin
        if !verbose then
          Format.printf "flushing %08x@." win;

        flush_events' win [PropertyNotify];
        select_input win []
      end

  | PropertyNotifyEvent e when
      e.P.state = PropertyNewValue &&
      e.P.atom = atom_wmstate
    ->
      let win = e.P.window in
      let state =
        try
          let state, icon = Icccm.getWM_STATE dpy win in
          Some state
        with Not_found -> None
      in

      if !verbose then begin
        let state =
          match state with
            Some state -> string_of_state state
          | None -> "nostate"
        in
        Format.printf "newstate %08x %s@." win state;
      end;

      let category = categorize_hints win in
      if category = `withdrawn then begin
        match state with
          Some state -> preadd_client win
        | None -> ()
      end

  | otherwise -> ()

let main () =
  select_input root root_input;
  let withdrawn_list = find_withdrawn root in
  start_clients ();
  List.iter preadd_client withdrawn_list;

  X.mapWindow dpy slit;
  X.grabButton dpy slit false
    [ButtonPressMask; ButtonReleaseMask; Button1MotionMask]
    GrabModeSync GrabModeAsync
    root noCursor 1 mod1Mask;

  hidden := false;
  reconfigure ();

  let rec run_forever mode =
    X.allowEvents dpy SyncPointer currentTime;

    let ev = nextEvent dpy in

    begin try
      if ev.ev_window = root then
        root_event ev
      else
        if ev.ev_window != slit then
          win_event ev
    with _ -> ()
    end;

    let mode =
      if ev.ev_window = slit then
        match ev.ev_event with
          EnterNotifyEvent en   -> handle_enter en mode
        | LeaveNotifyEvent ln   -> handle_leave ln mode
        | ButtonPressEvent bp   -> handle_buttonpress bp mode
        | ButtonReleaseEvent br -> handle_buttonrelease br mode
        | MotionNotifyEvent mn  -> handle_motion mn mode
        | UnmapNotifyEvent un   -> handle_unmap ev un mode
        | DestroyNotifyEvent dn ->
            maybe_remove_client true dn.Xdestroywindow.window;
            mode
        | _ -> mode
      else mode
    in
    run_forever mode
  in
  run_forever MNone

let sigfun signum =
  Format.eprintf "Got signal: %d@." signum;
  remove_clients ();
  closeDisplay dpy;
  exit 1

let _ =
  List.iter (fun signum -> Sys.set_signal signum (Sys.Signal_handle sigfun))
    [ Sys.sighup
    ; Sys.sigint
    ; Sys.sigquit
    ; Sys.sigsegv
    ; Sys.sigterm
    ; Sys.sigpipe
    ];
  Sys.set_signal Sys.sigchld Sys.Signal_ignore;
  begin try
    main ()
  with
    Exit -> if !verbose then Format.printf "Normal exit have a nice day@."
  | XError x -> Format.eprintf "Uncaught X exception:\n%s@." (xerror_to_string x)
  | e -> Format.eprintf "Uncaught exception:\n%s@." (Printexc.to_string e)
  end;
  remove_clients ();
  closeDisplay dpy
